/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2014 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */


package com.openexchange.spamsettings.generic.servlet;

import java.io.IOException;
import java.util.Map;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.json.JSONException;
import org.json.JSONObject;
import com.openexchange.ajax.PermissionServlet;
import com.openexchange.ajax.container.Response;
import com.openexchange.ajax.writer.ResponseWriter;
import com.openexchange.exception.OXException;
import com.openexchange.spamsettings.generic.SpamSettingsParser;
import com.openexchange.spamsettings.generic.SpamSettingsWriter;
import com.openexchange.spamsettings.generic.osgi.SpamSettingsServiceRegistry;
import com.openexchange.spamsettings.generic.service.SpamSettingService;
import com.openexchange.tools.servlet.OXJSONExceptionCodes;
import com.openexchange.tools.session.ServerSession;

/**
 * @author <a href="mailto:martin.herfurth@open-xchange.com">Martin Herfurth</a>
 */
public final class SpamSettingsServlet extends PermissionServlet {

    /**
     *
     */
    private static final long serialVersionUID = 2124511440962531967L;

    private static final org.slf4j.Logger LOG = org.slf4j.LoggerFactory.getLogger(SpamSettingsServlet.class);

    /**
     * Initializes
     */
    public SpamSettingsServlet() {
        super();
    }

    @Override
    protected boolean hasModulePermission(final ServerSession session) {
        return true;
    }

    @Override
    protected void doGet(final HttpServletRequest req, final HttpServletResponse resp) throws IOException {

        final Response response = new Response();

        final ServerSession session = getSessionObject(req);
        try {
            final String action = JSONUtility.checkStringParameter(req, "action");
            if ("get".equals(action)) {
                final JSONObject result = new JSONObject();
                result.put("formDescription", new SpamSettingsWriter().write(session));
                final SpamSettingService service = SpamSettingsServiceRegistry.getServiceRegistry().getService(SpamSettingService.class);
                result.put("value", getValue(session, service));
                response.setData(result);
            }
        } catch (final OXException e) {
            LOG.error("Missing or wrong field action in JSON request", e);
            response.setException(e);
        } catch (final JSONException e) {
            LOG.error("", e);
            response.setException(OXJSONExceptionCodes.JSON_WRITE_ERROR.create(e));
        }

        /*
         * Close response and flush print writer
         */
        try {
            ResponseWriter.write(response, resp.getWriter(), localeFrom(session));
        } catch (final JSONException e) {
            LOG.error(e.getLocalizedMessage(), e);
        }
    }

    private JSONObject getValue(final ServerSession session, final SpamSettingService service) throws JSONException, OXException {
        final Map<String, Object> settings = service.getSettings(session);
        final JSONObject retval = new JSONObject();
        for (final Map.Entry<String, Object> entry : settings.entrySet()) {
            retval.put(entry.getKey(), entry.getValue());
        }
        return retval;
    }

    @Override
    protected void doPut(final HttpServletRequest req, final HttpServletResponse resp) throws ServletException, IOException {

        final Response response = new Response();

        final JSONObject obj = new JSONObject();

        try {
            final String action = JSONUtility.checkStringParameter(req, "action");
            if (action.equals("update")) {
                final String body = getBody(req);
                final JSONObject jsonObject = new JSONObject(body);
                final Map<String, Object> settings = new SpamSettingsParser().parse(getSessionObject(req), jsonObject);
                final SpamSettingService spamSettingService = SpamSettingsServiceRegistry.getServiceRegistry().getService(SpamSettingService.class);
                spamSettingService.writeSettings(getSessionObject(req), settings);
                obj.put("message", "Settings written");
            }
        } catch (final OXException e) {
            LOG.error("Missing or wrong field action in JSON request", e);
            response.setException(e);
        } catch (final JSONException e) {
            LOG.error(e.getLocalizedMessage(), e);
            response.setException(OXJSONExceptionCodes.JSON_READ_ERROR.create(e));
        }

        response.setData(obj);

        /*
         * Close response and flush print writer
         */
        try {
            ResponseWriter.write(response, resp.getWriter(), localeFrom(getSessionObject(req)));
        } catch (final JSONException e) {
            LOG.error(e.getLocalizedMessage(), e);
        }
    }

}
